/*****************************************************************************
 * Copyright (C) 2004 Jean-Daniel Fekete and INRIA, France                   *
 * ------------------------------------------------------------------------- *
 * See the file LICENCE.TXT for license information.                         *
 *****************************************************************************/
package fr.inria.infovis.column;

import infovis.Column;
import infovis.column.AbstractColumn;
import infovis.column.BasicColumn;
import infovis.column.BooleanColumn;
import infovis.column.DoubleColumn;
import infovis.column.FloatColumn;
import infovis.column.IntColumn;
import infovis.column.LongColumn;
import infovis.column.ObjectColumn;
import infovis.column.StringColumn;
import infovis.utils.RowIterator;

import java.text.Format;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.event.ChangeListener;


/**
 * 
 * A proxy Column to handle the layer mechanism (i.e adding another dimension to a
 * standard 1D column).
 * <br/><b><i>Principle</i></b><br/>
 * This proxy contains a working column. This column is used as a frontend 
 * and then interacts directly with the rest of the Infovis classes.
 *   
 * Values are dispatched in several layer 
 * by using <code>setValueAt(int layer, int index, String element)</code>
 * When restoring a given layer, the working column is first entirely cleared and 
 * secondly values of the layers are copied into the column
 * 
 * Values storage is based on three structures 
 * <ul><li>The working column where the current layer values are stored. 
 * Its size correspond to the "real" size of the column</li>
 * <li> the storage column a backend column storing all the values in all layers. 
 * By the way it contains much more elements than the the frontend column... 
 * This backend column has the same type than the frontend Column. 
 * This ensures that this class can be used with "theorically" any type of Infovis Column.
 * But it is currently restricted to BasicColumn due to the fact that interface Column 
 * does not declare all the methods required.</li>
 * <li> a mapping table between backend and frontend column. It is constitued of a TreeMap
 * of IntVector. TreeMap keys are Integer representing the layers. 
 * IntVector are the modifications of the column for this layer. 
 * IntVector contents works with pair. Odd attributes are index in frontend column,
 * even attributes are index in backend column.</li></ul>     
 * 
 * 
 * @author raf
 */
public class LayeredColumnProxy extends AbstractColumn   implements LayeredColumn {
	
	
    /**
     * This <code>column</code> attributes represents the working column, i.e. the front end
     * column used by other objects (such as visualization)
     */
    protected BasicColumn column;
    /**
     * This <code>columnClass</code> attributes represents the class of the "decored" column
     */
    protected Class columnClass;
    
    protected Column values;
    
    /**
     * <code>aggregator</code> allows to combine values at insertion.
     * For instance, when inserting a value at a given index in a layer, 
     * this value can be combined with the value inserted in the last layer
     * at the same index. 
     * <i> this feature is experimental </i> 
     */
    protected ValueAggregator aggregator;
    /**
     * <code>aggregator</code> allows to combine values at restore.
     * For instance, when restoring values at a given index from a layer, 
     * this value can be combined with the current value restored by
     * a previous layer at the same index. 
     * <i> this feature is experimental </i> 
     */
    protected ValueAggregator restoreAggregator;
    /**
     * The <code>modifications</code> vector will store indexes of modified at layer i
     * This TreeMap uses integer as key representing the layers
     * The content of the TreeMap is IntVector using pairs. The first index is the
     * index of the cell in the frontend column to modify. The second index is the
     * index of the cell containing the value in the backend column.
     * <br/><i>Example<i><br/>
     * Integer(5), [1,20,12,30,15,60] means :<br/>
     * Int the layer #5 :<br/><ul>
     * <li>the cell #1 of the frontend column must use the value stored in the cell #20 of the backend column</li>
     * <li>the cell #12 of the frontend column must use the value stored in the cell #30 of the backend column</li>
     * <li>the cell #15 of the frontend column must use the value stored in the cell #60 of the backend column</li>  
     */
    protected TreeMap modifications;
    
    
   
    
    /**
     * 
     * Default decorator constructor. Build a new instance
     * of the layer mecanism attached to the given concrete column.
     * 
     * @param column the concrete/frontend column
     * 
     * NOTE: If the column already contains values at the creation, values ARE NOT copied
     * into an layer. It is strongly recomended to use this constructor with EMPTY column.
     */
    public LayeredColumnProxy(BasicColumn column) {
    	this.column = column;
    	this.columnClass = column.getClass();
    	String name = "#LAYERED_VALUES_"+column.getName();
    	
// 		Does not work because columns are abstracted and does not implements
//    	a default constructor
//  	try {
//  	this.values = (Column) this.columnClass.newInstance();
//  	} catch (InstantiationException e) {
//  	e.printStackTrace();
//  	} catch (IllegalAccessException e) {
//  	e.printStackTrace();
//  	}
//  	this.values.setName(name);
    	if (column instanceof BooleanColumn) {
    		values = new BooleanColumn(name);
    	} else if (column instanceof DoubleColumn) {
    		values = new DoubleColumn(name);
    	} else if (column instanceof FloatColumn) {
    		values = new FloatColumn(name);
    	} else if (column instanceof IntColumn) {
    		values = new IntColumn(name);
    	} else if (column instanceof LongColumn) {
    		values = new LongColumn(name);
    	} else if (column instanceof StringColumn) {
    		values = new StringColumn(name);
    	} else {
    		values = new ObjectColumn(name);
    	}
    	// No notification needed for the backend column
    	values.disableNotify();
    	modifications= new TreeMap();
    }
    

    /**
     * 
     * Clear the contents of the three underlaying structure 
     * (backend and frontend columns and the mapping TreeMap)
     * 
     * @see infovis.Column#clear()
     */
    public void clear() {
        if (column != null) column.clear();
        if (values != null) values.clear();
        if (modifications != null) modifications.clear();
    }
    

    /**
     * 
     * add a new layer for the column.
     * If the layer exists, it is not recreated.
     * 
     * @param layer the new layer to create
     * @return the InVector representing the modifications of the frotend columns.
     */
    protected IntVector addLayer(int layer) {    	
    	Integer key = new Integer(layer);
    	if (modifications.containsKey(key)) {
    		return (IntVector) modifications.get(key);
    	} else  {
    		IntVector result = new IntVector();
    		modifications.put(key, result);
    		return result;
    	}
    }
    
    
    /**
     * 
     * Restore all the values contained by layers <code>[from,to]</code> 
     * in the frontend column.
     * <ul>
     * <li> Reorder the layer bounds <code>from</code> and <code>to</code></li>
     * <li> Disable notifications on the frontend column </li>
     * <li> Reset the frontend column by setting undefined values </li>
     * <li> Exctract the Submap if any </li>
     * <li> Enumerates all the layers int the submap </li>
     * <li> Restore notification (performing event firing if necessary) </li>
     * 
     * This method is not very efficient. It restores entirely all the layer
     * It could be optimized if there is no combination of values of differents layers. 
     * 
     * @see fr.inria.infovis.column.LayeredColumn#restoreLayers(int, int)
     */
    public void restoreLayers(int from, int to) {   	
    	if (from > to) {
    		restoreLayers(to, from); //reoreder the bounds
    	} else {
    		//System.out.println("Restoring column \""+this.getName()+"\" from:"+from+" to:"+to);
    		if (!modifications.isEmpty()) {
    			column.disableNotify();    // disable notification of the frontend column
    			//System.out.println("Resetting column");
    			RowIterator rit = column == null ? null : column.iterator();
    			while(rit != null && rit.hasNext()) {
    				column.setValueUndefined(rit.nextRow(),true);
    			}
    			
    			Integer fromKey = new Integer(from);
    			Integer toKey = new Integer(to);
    			boolean fromStart = (fromKey.compareTo(modifications.firstKey()) <= 0);
    			boolean toEnd = (toKey.compareTo(modifications.lastKey())>= 0);
    			SortedMap inter; //the submap extracted from the modification map and corresponding to
    						     // the layers [from,to]
    			if (fromStart && toEnd) {
    				inter = modifications;
    			} else if (fromStart) {
    				inter = modifications.headMap(toKey);
    			} else if (toEnd) {
    				inter = modifications.tailMap(fromKey);
    			} else {
    				inter = modifications.subMap(fromKey, toKey);
    			}
    			if (inter == null || inter.isEmpty()) {
    				//System.out.println("Empty submap");
    			} else {
    				//System.out.println("Sub Map:"+inter.firstKey()+" to " + inter.lastKey());
    				IntVector layer;
    				Collection layers = inter.values();
    				Iterator it = (layers == null ? null : layers.iterator());
    				while (it != null && it.hasNext()) {
    					layer = (IntVector) it.next();
    					restoreLayer(layer);
    				}	
    			}
    			//System.out.println("Enable notification");
    			column.enableNotify();
    		}
    	}
    }
    

    /**
     * 
     * Restore the values contained in a particumlar layer.
     * The layer is seen as the IntVector representing modifications
     * of the frontend column in the layer.
     * 
     * @param layer  the modifications for the layer.
     */
    protected void restoreLayer(IntVector layer) {
    	if (layer != null) {
    		int index;
    		int value;
    		
    		for(int i=0; i < layer.size(); i+=2) {
    			index = layer.get(i);
    			value = layer.get(i+1);
    			
    			column.setValueOrNullAt(index, values.getValueAt(value));
    			column.setValueUndefined(index,false);
    		}
    	}
    }




    /**
     * 
     * This function is part of the standard Column Interface.
     * When using the Layer mecanism, it is necessary to give a layer
     * where to add the value. This method is unusable here.
     * So this method does noting else than producing a warning.
     * 
     * 
     * @see infovis.Column#setValueAt(int, java.lang.String)
     */
    public void setValueAt(int index, String element)
        throws ParseException {
    	System.err.println("Do not use setValueAt(int index, String element) with a Layered column");
    }

    
    
    /**
     * 
     * This method set the value of cell in the column into a particular layer.  
     * This value becomnes the current value in the frontend column for the cell <code>#row</code> 
     * 
     * 
     * @param layer the layer concerned by the modification
     * @param index the index of the cell concerned by the modification (in the frontend column)
     * @param element the new value to set in the cell,layer
     * @throws ParseException Element is given as a string. 
     * It must be parsed to be converted into int,float, etc...depending on the Column type.
     * If the conversion can not be done, this exception is thrown.
     */
    public void setValueAt(int layer, int index, String element) throws ParseException {
    	String value;
    	if (aggregator == null) {
    		value = element;  
    	} else {
    		if (column.isValueUndefined(index)) {
    			value = element;
    		} else {
    			String current = column.getValueAt(index);
    			value = aggregator.aggregate(current,element);
    		}
    	}
    	IntVector l = addLayer(layer);
    	values.addValue(element);
    	l.add(index);
    	l.add(values.lastValidRow());
    	column.setValueAt(index, value);	
    }
    
    
    /**
     * This method set the value of cell in the column into a particular layer.  
     * The difference with <code>setValueAt</code> is the possibilitiy to set a null value.
     * 
     * 
     * @param layer the layer concerned by the modification
     * @param index index the index of the cell concerned by the modification (in the frontend column)
     * @param element the new value to set in the cell,layer, can be <code>null<code>
     */
    public void setValueOrNullAt(int layer, int index, String element) {
    	String value;
    	if (aggregator == null) {
    		value = element;  
    	} else {
    		if (column.isValueUndefined(index)) {
    			value = element;
    		} else {
    			String current = column.getValueAt(index);
    			value = aggregator.aggregate(current,element);
    		}
    	}
    	IntVector l = addLayer(layer);
    	values.addValueOrNull(value);
    	l.add(index);
    	l.add(values.lastValidRow());
    	column.setValueOrNullAt(index, value);	
    }
    
    
    /**
     * Return the value of the cell currently set in the frontend column.
     * This value v
     * @see infovis.Column#getValueAt(int)
     */
    public String getValueAt(int row) {
    	String result = (column == null ? null : column.getValueAt(row));
    	return result;
    }
    

	/**
	 * @return Returns the aggregator for insertion (experimental)
	 */
	public synchronized ValueAggregator getAggregator() {
		return aggregator;
	}
	/**
	 * @param aggregator The insertion aggregator to set (experimental).
	 */
	public synchronized void setAggregator(ValueAggregator aggregator) {
		this.aggregator = aggregator;
	}
	/**
     * 
     * This function is part of the standard Column Interface.
     * When using the Layer mecanism, it is necessary to give a layer
     * where to add the value. This method is unusable here.
     * So this method does noting else than producing a warning.
     *  
	 * @see infovis.Column#addValue(java.lang.String)
	 */
	public void addValue(String v) throws ParseException {
		System.err.println("Do not use addValue(String v) with a Layered column");
	}
	/**
     * 
     * This function is part of the standard Column Interface.
     * When using the Layer mecanism, it is necessary to give a layer
     * where to add the value. This method is unusable here.
     * So this method does noting else than producing a warning.
     * 
	 * @see infovis.Column#addValueOrNull(java.lang.String)
	 */
	public boolean addValueOrNull(String v) {
		System.err.println("Do not use addValueOrNull(String v) with a Layered column");
		return false;
	}

	
	
	/**
	 * add a change listener for the frontend column.
	 * The backend column has bisabled its notification mechanism.
	 * 
	 * @param listener the changeListener to add
	 */
	public void addChangeListener(ChangeListener listener) {
		column.addChangeListener(listener);
	}
	/**
	 * @return the capapcity of the frontend Column
	 */
	public int capacity() {
		return column.capacity();
	}
	/**
	 * 
	 * compare two values at <code>row1</code> and <code>row2</code>
	 * in the frontend column. 
	 * 
	 * @param row1
	 * @param row2
	 * @return the result of the comparision
	 */
	public int compare(int row1, int row2) {
		return column.compare(row1, row2);
	}
	/**
	 * disable notification for the of the frontend column
	 */
	public void disableNotify() {
		column.disableNotify();
	}
	/**
	 * enable notification of the frontend column
	 */
	public void enableNotify() {
		column.enableNotify();
	}
	/**
	 * 
	 * ensure the capacitiy of the frontend column
	 * 
	 * @param minCapacity the capacity required
	 */
	public void ensureCapacity(int minCapacity) {
		column.ensureCapacity(minCapacity);
	}
	/**
	 * 
	 * Column equalities : comparison with the frontend column
	 * 
	 * @param obj  the object to compare with
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return column.equals(obj);
	}
	/**
	 * 
	 *
	 * @return the first valid row of the frontend column
	 */
	public int firstValidRow() {
		return column.firstValidRow();
	}
	/**
	 * @return the property map of the frontend column
	 */
	public Map getClientPropery() {
		return column.getClientPropery();
	}
	/**
	 * @return the format of the frontend column
	 */
	public Format getFormat() {
		return column.getFormat();
	}
	/**
	 * @return the metatdata map of the frontend column
	 */
	public Map getMetadata() {
		return column.getMetadata();
	}
	/**
	 * @return the name of the frontend column
	 */
	public String getName() {
		return column.getName();
	}
	/**
	 * @return the number of rows in the frontend column
	 */
	public int getRowCount() {
		return column.getRowCount();
	}
	/**
	 * @return the class of object stroed in the frontend column
	 */
	public Class getValueClass() {
		return column.getValueClass();
	}
	
	/**
	 * @return if the frontend column is empty
	 */
	public boolean isEmpty() {
		return column.isEmpty();
	}
	/**
	 * @return if the frontend column is an internal column
	 */
	public boolean isInternal() {
		return column.isInternal();
	}
	/**
	 * 
	 * Check if the value at <code>row</code> is
	 * undefined or not 
	 * 
	 * @param row the cell index intthe the frontend column
	 * @return if the value is defined or not
	 */
	public boolean isValueUndefined(int row) {
		return column.isValueUndefined(row);
	}
	/**
	 * @return an iterator on the rows of the frontend column
	 */
	public RowIterator iterator() {
		return column.iterator();
	}
	/**
	 * @return the last valid row number in the frontend column
	 */
	public int lastValidRow() {
		return column.lastValidRow();
	}
	/**
	 * @return the max value currently stored in the frontend column
	 */
	public String maxValue() {
		return column.maxValue();
	}
	/**
	 * @return the min value currently stroed in the frontend column
	 */
	public String minValue() {
		return column.minValue();
	}
	/**
	 * remove a Cahnge listenerfrom the frontend column
	 * 
	 * @param listener the listener to remove
	 */
	public void removeChangeListener(ChangeListener listener) {
		column.removeChangeListener(listener);
	}
	/**
	 * 
	 * Set the format for the frontend column
	 * @param format the format to set
	 */
	public void setFormat(Format format) {
		column.setFormat(format);
	}
	/**
	 * 
	 * Set the name of the frontend column
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		column.setName(name);
	}
	/**
	 * 
	 * Set a value in the frontend column
	 * 
	 * @param index
	 * @param v
	 * @return if the value has been set
	 * 
	 * IT IS NOT RECOMMENDED TO USE THIS METHOD
	 * Use setValueOrNullAt(int layer, int index, String v) instead.
	 */
	public boolean setValueOrNullAt(int index, String v) {
		return column.setValueOrNullAt(index, v);
	}
	/**
	 * 
	 * Set as undefined the value at row <code>#i</code>
	 * 
	 * @param i  the row to disable/enable
	 * @param undef make the value undefined or defined
	 */
	public void setValueUndefined(int i, boolean undef) {
		column.setValueUndefined(i, undef);
	}
	/**
	 * 
	 * @return a String representing the frontend column
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return column.toString();
	}
	/**
	 * @return Returns the frontend column
	 *
	 */
	public synchronized Column getColumn() {
		return column;
	}
	/**
	 * 
	 * Set the frontend column
	 * 
	 * @param c The column to set.
	 */
	public synchronized void setColumn(BasicColumn c) {
		this.column = c;
	}
	/** 
	 * 
	 * Set the frontend column 
	 * 
	 * @param c The column to set.
	 * 
	 * @see fr.inria.infovis.column.LayeredColumn#setColumn(infovis.Column)
	 */
	public void setColumn(Column c) {
		this.column = (BasicColumn)c;
	}
}
