/*
 * Created on 6 mai 2004
 *
 */
package fr.inria.infovis.temporizer;

import infovis.Table;
import infovis.Visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;




/**
 * 
 * Abstract Implementation of a Temporizer of notification for
 * Infovis objects like Graph, Table or Tree.
 * This class disable/enable notification mecanism periodically.
 * This is very useful when Infovis Object is attached to a visualization. 
 * The visualization is refreshed periodically.   
 * 
 * It works as Decorator of Table.
 * 
 * This method us a Swing Timer. a better implementation using java.util.Timer
 * is also possible.
 * 
 * @author raf
 *
 */
public abstract class AbstractTemporizer  implements ActionListener {
	/**
	 *  <code>DEFAULT_DELAY</code> period between two disable/enable notification
	 */
	public static int DEFAULT_DELAY=2000; //Two second
	/**
	 *  <code>LIMIT_DELAY</code> if the action will last more than this delay, the temporizer is stopped
	 */
	public static int LIMIT_DELAY=30000; //if action lasts more than this limit, Temproizer must be aborted;
	
	/**
	 *  <code>timer</code> The Timer used to periodically perfom the action
	 */
	protected Timer timer;
	/**
	 * <code>table</code> store a reference to the "temporized" table,tree or graph.
	 */
	protected Table table;
	/**
	 * <code>visualization</code> a refernce to a visualization attached to the "temporized" table, tree or graph
	 * This reference is useful to get the painting time and to adapt the delay of the timer.
	 * This is mandatory when painting is very long. If we not adapt the delay, the application 
	 * will only perform painting
	 */
	protected Visualization visualization;
	/**
	 *  <code>delay</code> The current delay to disable/enable notification.
	 * This delay is given by the user. But it may evolve due to particularly long action
	 * performed durring notification.
	 */
	protected int delay;
	
	/**
	 * 
	 * Construct a new temporizer for the infovis Object
	 * 
	 * @param t : the Infovis object "temporized"
	 */
	public AbstractTemporizer(Table t) {
		this(DEFAULT_DELAY, t);
	}
	
	/**
	 * 
	 * Construct a new temporizer for the infovis Object giving a delay
	 * 
	 * @param d the delay
	 * @param t the temporozed object
	 */
	public AbstractTemporizer(int d, Table t) {
		this.table = t;
		this.delay = d;
	}
	
	
	
	
	/**
	 * Starts the temporization
	 */
	public void start() {	
		disableNotify();
		if (timer == null) {
			timer = new Timer(delay, this);
			timer.setRepeats(true);
			timer.setCoalesce(true); //Very useful for long painting
		}
		timer.start();
	}
	
	
	/**
	 * Stops the temporization and restore the notification
	 */
	public void stop() {	
		if (timer != null && timer.isRunning()) timer.stop();
		enableNotify();
	} 
	
	
	
	
	/**
	 * Disable the notification mecanism on the Infovis Object.
	 * This method must be abstract because Table, Tree and Graph
	 * does not works the same way 
	 * 
	 */
	protected abstract void disableNotify(); 

	/**
	 * Enable the notification mecanism on the Infovis Object.
	 * This method must be abstract because Table, Tree and Graph
	 * does not works the same way 
	 */
	protected abstract void enableNotify();
	
	
	
	/**
	 * 
	 * Perform the Timer action, basically :
	 * <ol>
	 * <li> enable notification => firing pending events </li>
	 * <li> measure time to execute notification </li>
	 * <li> adapt the delay if necessary </li>
	 * <li> finally disable notification for next turn </li>
	 * </ol>
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		int newDelay;
		long time = System.currentTimeMillis();
		enableNotify(); // perform redrawing, can be painfully slow...
		time = System.currentTimeMillis() -time; // Time to perform enableNotify() (and supposed to fire event)
		if (visualization != null) time += visualization.getLayoutTime()+visualization.getRedisplayTime();
		//System.out.println("Visualization:"+visualization+" Action+Repaint time:"+time+" ms");
		if (time > delay) { // if the time to perform action is superior to the delay of the timer
				            // the delay of the timer must be adapted (enlarged) 
				            //the delay new delay is 30% more that the task performance....
			newDelay=(int)(time * 130 / 100);
			if (newDelay <= LIMIT_DELAY) { // if the delay is inferior to 30 s
				disableNotify();
				setDelay(newDelay); 
				//System.out.println("New Delay for Display Temporizer:"+newDelay+" ms");
				if (timer != null) timer.setDelay(newDelay);
			} else {
				//System.out.println("New Delay too long for Display Temporizer ("+newDelay+" ms) !!!");
				if (timer != null && timer.isRunning()) timer.stop();
			}
		} else {
			disableNotify();
		}
	}
	/**
	 * @return Returns the delay.
	 */
	public synchronized int getDelay() {
		return delay;
	}
	/**
	 * @param delay The delay to set.
	 */
	public synchronized void setDelay(int delay) {
		this.delay = delay;
	}
	/**
	 * @return Returns the graph.
	 */
	public synchronized Table getTable() {
		return table;
	}
	/**
	 * @param graph The graph to set.
	 */
	public synchronized void setTable(Table t) {
		this.table = t;
	}
	/**
	 * @return Returns the timer.
	 */
	public synchronized Timer getTimer() {
		return timer;
	}
	/**
	 * @return Returns the visualization.
	 */
	public synchronized Visualization getVisualization() {
		return visualization;
	}
	/**
	 * @param visualization The visualization to set.
	 */
	public synchronized void setVisualization(Visualization visualization) {
		this.visualization = visualization;
	}
}
