/*
* INRIA-OADYMPPAC
* 
* Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
* Author	: 	raf
*           	COSYTEC
*		    	4 rue Jean Rostand
* 				91893 Orsay Cedex, France
*				Tel. +33 1 60 19 37 38
*				Fax. +33 1 60 19 36 20
*
* Purpose	: COMPLETE HERE
* 
* Created on 10 mars 2004
*
* 
*/
package fr.inria.oadymppac.examples.parsing;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import fr.inria.oadymppac.io.GenericTraceFileFilter;
import fr.inria.oadymppac.io.PostMortemInputSource;
import fr.inria.oadymppac.parsing.gentra4cp.handlers.TraceContentHandler;
import fr.inria.oadymppac.utils.log.LogConfigurator;


/**
 * 
 * An example of a Parser Handler
 * Basically its only display the event.
 * 
 * @author raf
 *
 */
public class SingleParserExample {
    /**
     * <code>logger</code> a logger instance for this class
     */
    static Logger logger = Logger.getLogger(SingleParserExample.class);
    /**
     * <code>DEFAULT_FOLDER</code> where to find datafiles
     */
    protected final static File DEFAULT_FOLDER=new File("data");
    /**
     * <code>parser</code> The parser for the file
     */
    SAXParser parser;
    /**
     * <code>reader</code> The reader associated to the parser
     */
    XMLReader reader;
    /**
     * <code>defaultHandler</code> a default handler to manage errors, DTD and so on
     */
    DefaultHandler defaultHandler;
    /**
     * <code>mainHandler</code> a trace handler to analyse the GENTRA4CP file
     * It is preconfigured to uses the next listener;
     */
    TraceContentHandler mainHandler;  
    
    /**
     * <code>headListener</code> a header listener to analyse the HEADER of the GENTRA4CP file
     * The main Handler will delagate Header analysis to this handler
     */
    SingleHeadEventListener headListener;
    /**
     *  <code>providableListener</code> a provide listener to analyse the provide inside the header.
     * The Header listener will delegate the analysis of the provide to this handler
     * 
     * <code>providedListener</code> a provide listener to analyse the provide inside the document.
     * The trace listener will delegate the analysis of the provide to this handler
     */
    SingleProvideEventListener providableListener, providedListener;
    /**
     * <code>controlListener</code> a control listener to analyse Packet and Breakpoint.
     * It is generally tranparent and the analysis is redirected and performend by the mainHandler
     */
    SingleControlEventListener controlListener;
    /**
     * <code>complementListener</code> a complement listener to analyse Complement.
     * It is generally tranparent and the analysis is redirected and performend by the mainHandler
     */
    SingleComplementEventListener complementListener;
    /**
     * <code>solverListener</code> a Listener to handle GENTRA4CP tags
     */
    SingleSolverEventListener solverListener;
  
    
    /**
     * Default constructor
     */
    public SingleParserExample() {
        logger.info("Building new instance of the application");
        init_handlers(); // init all the handlers and listeners
    };
    
    /**
     * 
     * Start the application 
     * 
     * @param args : command line arguments
     */
    public static void main(String[] args) {
        LogConfigurator.configure(); // configure the logger
        logger.info("Starting example program");
        SingleParserExample example = new SingleParserExample(); // create a new Single Parser Example
        logger.info("Number of arguments:"+args.length);
        if (args.length < 1) // if there is no argument, open a file selector, else use the argument as filename.
            example.selectFile();
        else
            example.parse(args[args.length-1]);
    }
    
    
    /**
     * 
     * Use a file selector to help the user in choosing a file
     * When chosen, start parsing
     * 
     */
    protected void selectFile() {
    	JFileChooser fileChooser = new JFileChooser();
    	fileChooser
        .setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

    	fileChooser.setCurrentDirectory((DEFAULT_FOLDER.exists() ? DEFAULT_FOLDER : new File(".")));
    	fileChooser.setFileFilter(new GenericTraceFileFilter());
    	fileChooser.setDoubleBuffered(false);

    	int ret = fileChooser.showOpenDialog(null);
    	if (ret == JFileChooser.APPROVE_OPTION) {
    		 parse(fileChooser.getSelectedFile().getAbsolutePath());
    	} 
    }
    
    /**
     * 
     * Parse a GENTRA4CP file 
     * 
     * @param filename : the file to parse
     */
    protected void parse(String filename){
        if (logger.isDebugEnabled()) logger.info("Parsing XML file:"+filename);
        try {
            InputSource is=new PostMortemInputSource(filename); // create a GENTRA4CP input source
            parser = SAXParserFactory.newInstance().newSAXParser(); // use the JAVA SAX Framework to get a Parser 
            defaultHandler=new DefaultHandler(); // create a SAX Default Handler
            logger.info("New SAXParser created");
            reader = parser.getXMLReader();  // use the parser as a factory for the XML Reader (JAXP framework)
            reader.setContentHandler(mainHandler); // The content of the document is mainly handler by mainHandler
            	                                   // this handler will then delegate some part of the document to
                                                   // other handlers and listeners.
            reader.setDTDHandler(defaultHandler); // DefaultHandler will manage the DTD
            reader.setEntityResolver(defaultHandler); // DefaultHandler will manage the external entities
            reader.setErrorHandler(defaultHandler); // DefaultHandler will manage the SAX errors
            reader.parse(is); // Do parsing
            System.out.println("End of Parsing");
            
        } catch (SAXException e) {
            logger.error("SAX Exception",e); 
        } catch (FileNotFoundException e) {
            if (logger.isDebugEnabled()) {
                logger.error("FILE: " + filename + " cannot be found", e);
            }
        } catch (IOException e) {
            logger.error("I/O Exception",e);
        } catch (ParserConfigurationException e) {
            logger.error("Parser Configuration Exception",e);
        } finally {
            logger.info("Finally clause");
        }
        System.out.println("PROVIDED:"+ mainHandler.getHeadProvideInfo()); // show content of the Header
        solverListener.show(); // Show the stored events
        JOptionPane.showMessageDialog(null,"Parsing is ended successfully");
    }
    
    protected void init_handlers() {
        logger.info("Initializing default handler");
        mainHandler = new TraceContentHandler(); // declare a new TraceHandler
        
        headListener = new SingleHeadEventListener(); //declare a ne Header Listener 
        providableListener = new SingleProvideEventListener(SingleProvideEventListener.HEAD); //Declare a new Provide listener for header provide
        providedListener = new SingleProvideEventListener(SingleProvideEventListener.TRACE); // decalre a new provide listener for document provide
        controlListener = new SingleControlEventListener(); //declare a new control listener
        complementListener = new SingleComplementEventListener(); //declare a new complement listener
        solverListener = new SingleSolverEventListener(); // declare a new GENTRACP content listener
        
        
        
        mainHandler.addDefaultListener(solverListener); // attach the GENTRA4CP listener to the trace Handler
        
        mainHandler.addControlListener(controlListener); // attach the control Listener to the trace handler
        
        mainHandler.addHeadListener(headListener); // attach the header listener to the trace handler
        
        mainHandler.addProvidableListener(providableListener); // attach the header provide listener to the trace handler
        mainHandler.addProvidedListener(providedListener); // attach the document provide listener to the trace handler
        mainHandler.addComplementListener(complementListener); // attach the complement listener to the trace handler
     }
    
    
    
}
