/**
 * Copyright (C) 2003 Jean-Daniel Fekete and INRIA, France
 * -------------------------------------------------------------------------
 * This software is published under the terms of the QPL Software License a
 * copy of which has been included with this distribution in the
 * license-infovis.txt file.
 */
package fr.inria.oadymppac.panel;


import infovis.Graph;
import infovis.Table;
import infovis.Tree;
import infovis.Visualization;
import infovis.column.StringColumn;
import infovis.graph.visualization.GraphVisualization;
import infovis.graph.visualization.MatrixVisualization;
import infovis.panel.DefaultAction;
import infovis.table.visualization.ScatterPlotVisualization;
import infovis.tree.visualization.TreeVisualization;
import infovis.tree.visualization.TreemapVisualization;
import infovis.tree.visualization.treemap.SliceAndDice;
import infovis.visualization.VisualizationFactory;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import fr.inria.infovis.panel.ControlPanelFactory;
import fr.inria.infovis.panel.DefaultControlPanel;
import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.utils.Properties;

/**
 * Component to create a visualization program as simply as possible.
 *
 * @author Jean-Daniel Fekete
 * @version $Revision: 1.4 $
 */
public class VisualizationFrameDecorator  implements WindowListener {
	Properties properties;
	
    MainActionFactory actionFactory;
    
    JFrame frame;
    JMenuBar menuBar;
    JMenu fileMenu;
    JMenu analyseMenu;
    JMenu windMenu;
    JMenu helpMenu;
    
    JDesktopPane desktop;
    
    JSplitPane splitter;
    JPanel splitLeft;
    
    
    JMenu visualizationMenu;
    Table table;
    Visualization visualization = null;
    DefaultControlPanel controlPanel;
    
    ControlPanelFactory factory;
    
    JComponent historyController;
    
    
    
 
    /**
     * Creates a new MainFrameDecorator object.
     *
     * @param frame the JFrame to decorate.
     */

    public VisualizationFrameDecorator(Properties props) {
    	this.properties = props;
    	this.actionFactory = (MainActionFactory) properties.getProperty(ApplicationProperties.ACTION_FACTORY);
        setActionFactory(actionFactory);
        this.frame = (JFrame) properties.getProperty(ApplicationProperties.MAIN_FRAME);
    
        menuBar = frame.getJMenuBar();
        menuBar = (menuBar == null ? new JMenuBar() : menuBar);
        frame.setJMenuBar(menuBar);
        createMenus();
        AbstractAction action= actionFactory.getDefaultAction();
        action.actionPerformed(null);
        frame.addWindowListener(this);
    }

    /**
     * Creates the menus.
     */
    protected void createMenus() {
        Component[] submenus = menuBar.getComponents();
        fileMenu = null;
        for (int i=0; i < submenus.length; i++) {
            if (submenus[i] instanceof JMenu) {
                JMenu submenu = (JMenu)submenus[i];
                if (submenu.getText().equals(MainFrameDecorator.FILE_MENU)) fileMenu = submenu;
                else if (submenu.getText().equals("Analyse")) analyseMenu = submenu;
                else if (submenu.getText().equals("Visualization")) visualizationMenu = submenu;
                else if (submenu.getText().equals(MainFrameDecorator.WIND_MENU)) windMenu = submenu;     
                else if (submenu.getText().equals(MainFrameDecorator.HELP_MENU)) helpMenu = submenu;              
            }
        }

        createFileMenu();
        menuBar.add(fileMenu);
        createAnalyseMenu();
        menuBar.add(analyseMenu);
        createVisualizationMenu();
        menuBar.add(visualizationMenu);
        createWindMenu();
        menuBar.add(windMenu);
        createHelpMenu();
        menuBar.add(helpMenu);
    }

    /**
     * Creates the file menu.
     *
     * @return the file menu.
     */
    public void createFileMenu() {
        fileMenu = (fileMenu==null ? new JMenu(MainFrameDecorator.FILE_MENU):fileMenu);
        fileMenu.add(actionFactory.createOpenAction());
        fileMenu.addSeparator();
        fileMenu.add(actionFactory.createQuitAction());
    }

 
    
    
    public void createAnalyseMenu() {
    	analyseMenu = (analyseMenu == null ? new JMenu("Analyse"): analyseMenu);
        analyseMenu.add(actionFactory.createAnalyseAction());
        analyseMenu.setEnabled(false);
    }
    
    public void enableAnalyseMenu() {
    	analyseMenu.setEnabled(true);
    }
    
    public void disableAnalyseMenu() {
    	analyseMenu.setEnabled(false);
    }
    
    
    public void createVisualizationMenu() {
        visualizationMenu = (visualizationMenu == null ? new JMenu("Visualization") : visualizationMenu);
    	visualizationMenu.setEnabled(false);
    }
    /**
     * Creates the window menu.
     *
     * @return the window menu.
     */
    public void createWindMenu() {
        windMenu = (windMenu == null ? new JMenu(MainFrameDecorator.WIND_MENU): windMenu);
        windMenu.add(actionFactory.createTileHorizontallyAction());
        windMenu.add(actionFactory.createTileVerticallyAction());
        windMenu.add(actionFactory.createMosaicAction());
        windMenu.add(actionFactory.createCascadeAction());
        windMenu.addSeparator();
        windMenu.add(actionFactory.createZoomInAction());
        windMenu.add(actionFactory.createZoomOutAction());
        
    }
    
    
    /**
     * Creates the help menu.
     *
     * @return the help menu.
     */
    public void createHelpMenu() {
        helpMenu = (helpMenu == null ? new JMenu(MainFrameDecorator.HELP_MENU): helpMenu);
        helpMenu.add(actionFactory.createAboutAction());
        helpMenu.add(actionFactory.createSplashAction());
    }

    /**
     * DOCUMENT ME!
     *
     * @param t DOCUMENT ME!
     * @param index DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public static StringColumn getStringColumn(Table t, int index) {
        StringColumn ret = null;
        for (int i = 0; i < t.getColumnCount(); i++) {
            ret = StringColumn.getColumn(t, i);
            if (ret != null && !ret.isInternal() && index-- == 0)
                return ret;
        }
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @param table DOCUMENT ME!
     * @param visualization DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Visualization createControls(
        Table t,
        final  Visualization vis, final String name) {
    	if (splitter==null) {
    		splitter= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    		splitter.setSize(800,600);
    		splitter.setPreferredSize(new Dimension(800,600));
    		splitter.setDividerLocation(.50);
    		splitLeft =  new JPanel();
    		splitLeft.setLayout(new BorderLayout());
    		splitter.add(splitLeft,JSplitPane.LEFT );
    	}
    	if (historyController == null) {
    		historyController = new HistoryController() ;
    	}
    	
    	
       	if (desktop==null) {
    		desktop= new JDesktopPane();
    		desktop.setSize(500,500);
    		desktop.setPreferredSize(new Dimension(500,500));
    	}
       	
    	if (splitter != null) {
    		splitLeft.add(desktop,BorderLayout.CENTER);
    		splitLeft.add(historyController, BorderLayout.SOUTH);
    	}
    	
        if (factory == null) {
            factory = ControlPanelFactory.sharedInstance();
        }

         DefaultControlPanel control =
            (DefaultControlPanel) factory.createControlPanel(vis);
         
           VisualizationInternalFrame window = new VisualizationInternalFrame(properties, vis);
           window.setProperties(properties);
           window.setTitle(window.getTitle()+":"+name);  
           window.setSplitter(splitter);
           window.setHistoryController(historyController);
           
           
           desktop.add(window);
           
           int dl = splitter.getDividerLocation();   
        if (control != null) {
        	if (controlPanel != null) splitter.remove(controlPanel);
        	controlPanel = control;
        	splitter.add(controlPanel, JSplitPane.RIGHT);
        	window.setControlPanel(controlPanel);
        }
        splitter.setDividerLocation(dl);
 
        
        
 
        System.gc();
        System.runFinalization();
  
        VisualizationFactory.Creator[] creators =
            VisualizationFactory
                .sharedInstance()
                .getCompatibleCreators(
                t);
        visualizationMenu.removeAll();
        for (int i = 0; i < creators.length; i++) {
            addVisualizationMenu(creators[i], i);
        }
        visualizationMenu.setEnabled(creators.length != 0);
        
        
        
        
        if (frame != null) {
    		frame.getContentPane().add(splitter);
    		
    		System.out.println("VISUALIZATION ADDED");
    		frame.invalidate();
    		frame.show();
    		window.pack();
            window.setVisible(true);
			frame.repaint();
    	}
        
        
        return vis;
    }

    public void addVisualizationMenu(
        final VisualizationFactory.Creator creator,
        int i) {
        final String name = creator.getName();
        final Table t = this.table;
        visualizationMenu.add(new DefaultAction(name, '1' + i) {
            public void actionPerformed(ActionEvent e) {
                Visualization vis =creator.create(t);
                 createControls(t, vis, name);
            }
           
        });
    }

    /**
     * DOCUMENT ME!
     *
     * @param graph DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public GraphVisualization createGraphVisualization(Graph graph) {
        this.table = graph;
        MatrixVisualization vis = 
            new MatrixVisualization(graph) ;
        VisualizationFactory.Creator[] creators =
            VisualizationFactory
                .sharedInstance()
                .getCompatibleCreators(
                this.table);
        createControls(this.table, vis, creators[1].getName());
        return vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param tree DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public TreeVisualization createTreeVisualization(Tree tree) {
        this.table = tree;
        TreemapVisualization vis =
            new TreemapVisualization(
                tree,
                null,
                SliceAndDice.SLICEANDDICE) ;
        VisualizationFactory.Creator[] creators =
            VisualizationFactory
                .sharedInstance()
                .getCompatibleCreators(
                tree);
        createControls(tree, vis,creators[1].getName());
        return vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param table DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Visualization createTableVisualization(Table t) {
        this.table = t;
        Visualization vis =
            new ScatterPlotVisualization(table) ;
        VisualizationFactory.Creator[] creators =
            VisualizationFactory
                .sharedInstance()
                .getCompatibleCreators(
                table);
        createControls(table, vis,creators[1].getName());
        return vis;
    }

   
    /**
     * @return Returns the actionFactory.
     */
    public MainActionFactory getActionFactory() {
        return actionFactory;
    }
    /**
     * @param actionFactory The actionFactory to set.
     */
    public void setActionFactory(MainActionFactory actionFactory) {
        this.actionFactory = actionFactory;
    }
    
    
    
    
    
    
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
		
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		MainActionFactory actFactory = (MainActionFactory) properties.getProperty(ApplicationProperties.ACTION_FACTORY);
		if (actionFactory !=null) actFactory.quitAction.actionPerformed(null);
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
	}
	
	
	/**
	 * @return Returns the analyseMenu.
	 */
	public synchronized JMenu getAnalyseMenu() {
		return analyseMenu;
	}
	/**
	 * @return Returns the controlPanel.
	 */
	public synchronized DefaultControlPanel getControlPanel() {
		return controlPanel;
	}
	/**
	 * @return Returns the desktop.
	 */
	public synchronized JDesktopPane getDesktop() {
		return desktop;
	}
	/**
	 * @return Returns the historyController.
	 */
	public synchronized JComponent getHistoryController() {
		return historyController;
	}
	/**
	 * @return Returns the splitter.
	 */
	public synchronized JSplitPane getSplitter() {
		return splitter;
	}
}
