/*
 * $Id: EMNCountingBitSet.java,v 1.3 2004/05/28 06:28:48 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import java.util.BitSet;

/**
 * A <code>BitSet</code> that maintains the count of bits set to 1.
 *
 * @author Jean-Daniel Fekete
 * @version $Revision: 1.3 $
 */
public class EMNCountingBitSet extends BitSet {
    protected int count;

    /**
     * Creates a new counting bit set. All bits are initially <code>false</code>.
     */
    public EMNCountingBitSet() {
	count = 0;
    }

    /**
     * Creates a counting bit set whose initial size is large enough to explicitly
     * represent bits with indices in the range <code>0</code> through
     * <code>nbits-1</code>. All bits are initially <code>false</code>. 
     *
     * @param     nbits   the initial size of the bit set.
     * @exception NegativeArraySizeException if the specified initial size
     *               is negative.
     */
    public EMNCountingBitSet(int nbits) {
	super(nbits);
    }

    /**
     * Copy a counting bit.
     */
    public EMNCountingBitSet(EMNCountingBitSet other) {
	or(other);
    }
    
    /**
     * Sets the bit specified by the index to <code>false</code>.
     *
     * @param     bitIndex   the index of the bit to be cleared.
     * @exception IndexOutOfBoundsException if the specified index is negative.
     * @since     JDK1.0
     */
    public void clear(int bitIndex) {
	if (get(bitIndex)) {
	    super.clear(bitIndex);
	    count--;
	}
    }

    /**
     * Sets the bit specified by the index to <code>true</code>.
     *
     * @param     bitIndex   a bit index.
     * @exception IndexOutOfBoundsException if the specified index is negative.
     * @since     JDK1.0
     */
    public void set(int bitIndex) {
	if (! get(bitIndex)) {
	    super.set(bitIndex);
	    count++;
	}
    }

    public int getTrueCount() {
	return count;
    }

    protected void updateCount() {
	count = 0;
	for (int bit = 0; bit < size(); bit++) {
	    if (get(bit))
		count++;
	}
    }

    public void and(BitSet set) {
	super.and(set);
	updateCount();
    }
    public void or(BitSet set) {
	super.or(set);
	updateCount();
    }
    public void andNot(BitSet set) {
	super.andNot(set);
	updateCount();
    }
    public void xor(BitSet set) {
	super.xor(set);
	updateCount();
    }
}
