package fr.inria.oadymppac.solver.analyser;

import infovis.Graph;
import infovis.column.StringColumn;
import fr.inria.oadymppac.solver.Constraint;
import fr.inria.oadymppac.solver.ConstraintProxy;
import fr.inria.oadymppac.solver.Reduce;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.SolverException;
import fr.inria.oadymppac.solver.Variable;
import fr.inria.oadymppac.solver.VariableProxy;
import fr.inria.oadymppac.utils.DynamicIntArray;

/**
 * @author raf
 *
 * 
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class VariableConstraintGraphAnalyser extends AbstractGraphAnalyser implements SolverAnalyser {
	
	public static final String LABEL="Variable-Constraint";
	private static final int[] TYPES = {Solver.VARIABLE, Solver.CONSTRAINT, Solver.REDUCE};
	

	DynamicIntArray variableToNode;
	DynamicIntArray constraintToNode;

	
	StringColumn variableColumnEdge;
	StringColumn constraintColumnEdge;
	
	public void add(Variable variable) {
		try {   		
			if (logger.isDebugEnabled()) { 
				logger.info("Adding a new variable vertex:");
				logger.info("\t"+variable);
			}
			if (variableToNode== null) variableToNode = new DynamicIntArray();
			if (graph == null) init_graph();
			int node =graph.addVertex();
			variableToNode.set(variable.getIndex(),node);	
			if (logger.isDebugEnabled()) logger.info("adding at "+variable.getIndex()+" =>"+variableToNode);
			int chrono = variable.getChrono();
			int depth = variable.getDepth();
			chronoColumnNode.setExtend(node,chrono);
			invChronoColumnNode.setExtend(node,-chrono);
			depthColumnNode.setExtend(node,depth);
			invDepthColumnNode.setExtend(node,-depth);
			identifierColumnNode.setExtend(node,variable.getIdentifier());
			labelColumnNode.setExtend(node,variable.getLabel());
			nameColumnNode.setExtend(node,variable.getName());
			typeColumnNode.setExtend(node,variable.getEventType());
			numTypeColumnNode.setExtend(node,Solver.VARIABLE);
			internalColumnNode.setExtend(node,variable.getInternal());
			externalColumnNode.setExtend(node,variable.getExternal());
			activationColumnNode.setValueAt(chrono,node, "0");
			invActivationColumnNode.setExtend(node, 0);
			metricColumnNode.setValueAt(chrono,node, "0");
			invMetricColumnNode.setExtend(node, 0);
			
		} catch (Exception exp) {
			solver.add(Solver.EXCEPTION,exp);
		}
		
	}
	
	public void add(Constraint constraint) {
		try {		
			if (logger.isDebugEnabled()) logger.info("Adding a new constraint vertex corresponding to "+constraint.getIdentifier()+" at chrono="+constraint.getChrono());
			if (constraintToNode== null) constraintToNode = new DynamicIntArray();
			if (graph == null) init_graph();
			int node= graph.addVertex();
			constraintToNode.set(constraint.getIndex(),node);	
			if (logger.isDebugEnabled()) logger.info("adding at "+constraint.getIndex()+" => "+constraintToNode);
			int chrono = constraint.getChrono();
			int depth = constraint.getDepth();
			chronoColumnNode.setExtend(node,chrono);
			invChronoColumnNode.setExtend(node,-chrono);
			depthColumnNode.setExtend(node,depth);
			invDepthColumnNode.setExtend(node,-depth);
			identifierColumnNode.setExtend(node,constraint.getIdentifier());
			labelColumnNode.setExtend(node,constraint.getLabel());
			nameColumnNode.setExtend(node,constraint.getName());
			typeColumnNode.setExtend(node,constraint.getEventType());
			numTypeColumnNode.setExtend(node,Solver.CONSTRAINT);

			internalColumnNode.setExtend(node,constraint.getInternal());
			externalColumnNode.setExtend(node,constraint.getExternal());
			activationColumnNode.setValueAt(chrono, node, "0");
			invActivationColumnNode.setExtend(node, -0);
			metricColumnNode.setValueOrNullAt(chrono, node, "0");
			invMetricColumnNode.setExtend(node, -0);
			
		} catch (Exception exp) {
			solver.add(Solver.EXCEPTION, exp);
		}		
	}
	
	public void add(Reduce reduce) {
		buildWeightCalculator();
		Variable var =  reduce.getVariable();
		Constraint ctr =  reduce.getConstraint();
		if (var == null || ctr == null || (var instanceof VariableProxy) || (ctr instanceof ConstraintProxy) ) {
			solver.add(Solver.EXCEPTION, new SolverException("INVALID REDUCE REFERENCES AT "+reduce.getChrono()));
		} else {
			int metric = reduceWeightCalculator.getWeight(reduce);
			int chrono = reduce.getChrono();
			int depth = reduce.getDepth();
			int edge= add(chrono, reduce.getIndex(),var, ctr, metric);
			if (edge != Graph.NIL) {
				chronoColumnEdge.setExtend(edge,chrono);
				invChronoColumnEdge.setExtend(edge,chrono);
				depthColumnEdge.setExtend(edge,depth);
				invDepthColumnEdge.setExtend(edge,-depth);
				labelColumnEdge.setExtend(edge,reduce.getLabel());
				typeColumnEdge.setExtend(edge,reduce.getEventType());
				variableColumnEdge.setExtend(edge,var.getLabel());
				constraintColumnEdge.setExtend(edge,ctr.getLabel());
				numTypeColumnEdge.setExtend(edge,Solver.REDUCE);
			}
		}
	}
	
	
	
	protected int add(int chrono, int index, Variable variable, Constraint constraint, int metric) {
		try {   		
				int varNode = (variableToNode== null ? DynamicIntArray.NONE : variableToNode.get(variable.getIndex()));
				int ctrNode = (constraintToNode== null ? DynamicIntArray.NONE : constraintToNode.get(constraint.getIndex()));
				if (varNode != DynamicIntArray.NONE && ctrNode !=DynamicIntArray.NONE) {
					return addEdge(chrono, index, ctrNode, varNode, metric);
				} else  {
					if (logger.isDebugEnabled()) {
						logger.warn("Edge bounds not found");
						logger.warn("VARIABLE("+variable.getIdentifier()+",index="+variable.getIndex()+",node="+varNode+")"+
								"- CTR("+constraint.getIdentifier()+",index="+constraint.getIndex()+",node="+ctrNode+")");
						logger.warn("VARIABLE->NODE TABLE:"+variableToNode);
						logger.warn("CONSTRAINT->NODE TABLE:"+constraintToNode);
					}
				}	
		} catch (Exception exp) {
			solver.add(Solver.EXCEPTION,exp);
		}
		return Graph.NIL;
	}
	
	
	
	
	protected int[] getAnalyseTypes() {
		return  TYPES;
	}
	/**
	 * @param solver
	 */
	public VariableConstraintGraphAnalyser(Solver solver) {
		super(solver);
		
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.solver.analyser.SolverAnalyser#endAnalyse()
	 */
	public void init_graph() {
		super.init_graph();	
		variableColumnEdge = new StringColumn("Variable");
		edgeTable.addColumn(variableColumnEdge);
		constraintColumnEdge = new StringColumn("Constraint");
		edgeTable.addColumn(constraintColumnEdge);
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.solver.analyser.SolverAnalyser#getLabel()
	 */
	public String getLabel() {
		return LABEL;
	}
}


