/*****************************************************************************
 * Copyright (C) 2003 Jean-Daniel Fekete and INRIA, France                   *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the QPL Software License    *
 * a copy of which has been included with this distribution in the           *
 * license-infovis.txt file.                                                 *
 *****************************************************************************/
package fr.inria.oadymppac.utils;



public class DynamicIntArray  implements Cloneable {
	/**
	 * Comment for <code>DEFAULT_EXTEND</code>
	 * This defines the default extend of the array.
	 * Basically each time the DynamicIntArray needs to be extended
	 * its capacity is incremented. This is the default increment
	 */
	private static final int DEFAULT_EXTEND = 10;
	/**
	 * Comment for <code>NONE</code>
	 * This defines the NONE value for the DynamicArrayList. 
	 * This particular value is used to mark removed element
	 * or to fill the unused space of the array
	 */
	public static final int NONE = Integer.MIN_VALUE;
	/**
	 * Comment for <code>count</code>
	 * This attribute represents  the number of element different
	 * from NONE
	 */
	private int count;
	/**
	 * Comment for <code>extend</code>
	 * This represents the current value of capacity increment.
	 */
	private int extend;
	/**
	 * Comment for <code>latest</code>
	 * This attribute represents  the greatest index of element different
	 * from NONE
	 */
	private int last;
	/**
	 * Comment for <code>values</code>
	 * Contents of the vector;
	 */
	private int[] values;
	
	/**
	 * build a default dynamic integer array :
	 *  The initial capacity is set to DEFAULT_EXTEND
	 * 	The default block size for extension is set to the DEFAULT_EXTEND
	 */
	public DynamicIntArray() {
		this(DEFAULT_EXTEND, DEFAULT_EXTEND);
	}
	
	/**build a new dynamic integer array :
	 * @param reserve : the initial capacity of the array
	 * The default block size for extension is set to the DEFAULT_EXTEND
	 */
	public DynamicIntArray(int reserve) {
		this(reserve, DEFAULT_EXTEND);
	}
	
	/**build a new dynamic integer array :
	 * @param reserve = the initial capacity
	 * @param extend = the default block size for extension
	 */
	public DynamicIntArray(int reserve, int extend) {
		setExtend(extend);
		setValues(new int[(reserve == 0) ? extend : reserve]);
		clear(values);
		setCount(0);
		setLast(0);
	}
	
	/**build a new dynamic integer array with predefined values
	 * @param values : the intial values of this array;
	 */
	public DynamicIntArray(int[] values) {
		setValues(values);
		update();
	}
	
	/**
	 * Append a new element at the end of the array
	 * @param element : the integer appended
	 */
	public synchronized void add(int element)  {
		set(last + 1, element);
	}
	
	/**
	 * Append a set a of new elements at the end of the array
	 * @param elements : elements to appentd
	 */
	public synchronized void add(int[] elements)  {
		insert(last + 1, elements);
	}
	
	/**
	 * Insert into the array a set a of new elements 
	 * at the given index 
	 * @param elements : elements to append
	 * NOTE: The previous constant is first shifted 
	 * and then elements are inserted 
	 */
	public synchronized void insert(int index, int[] elements) {
		if (elements != null) {
			int shift=index+elements.length;
			ensureCapacity(shift);
			if (index <= last) System.arraycopy(values,index,values,shift,last-index+1);
			System.arraycopy(elements,0,values,index,elements.length);
			last+=elements.length;
			//count-=elements.length;
			for(int i=index; i < shift; i++) {
				if (values[i] != NONE) count++;
			}
		}
	}
	
	
	/**
	 * Clear the array : the content is reset but the array remains usable
	 * Its capacity remains avalilable to store new elements.
	 * 
	 */
	public final synchronized void clear() {
		clear(values) ;
		count=0;
		last=0;
	}
	
	
	
	/**
	 * Clear the value of the array at index
	 * @param index : the index to be cleared;
	 * @return the cleared value
	 */
	public synchronized int clear(int index) {
		int result = values[index]; 
		values[index] = NONE;
		count--;
		if (index == last) last--;
		return result;
	}
	
	/**
	 * Clear the values contained by the array between
	 * index from and index to; the rest of the array is not 
	 * @param array : the internal int array to reset;
	 */
	public synchronized void clear(int from, int to) {
		if (from > to) {
			clear(to,from);
		} else {
			if (to  > last)  {
				to = last;
				last = Math.max(0,from - 1) ;
			}
			for (int i = from; values != null &&  i <= to; i++) {
				values[i] = NONE;
				count--;
			}
			while(values[last]==NONE && last >0) last--; 
		}
	}
	/**
	 * Clear the values contained by the array
	 * @param array : the  int array to reset;
	 */
	protected synchronized void clear(int[] array) {
		for (int i = 0; i < array.length; array[i++] = NONE);
	}
	
	
	/**
	 * Clone the Array (i.e. build a copy) 
	 * @see java.lang.Object#clone()
	 * The content is also cloned
	 */
	public final Object clone() throws CloneNotSupportedException {
		int[] clonedValues = new int[values.length];
		System.arraycopy(values, 0, clonedValues, 0, values.length);
		DynamicIntArray clone = new DynamicIntArray(clonedValues);
		clone.setCount(this.count);
		clone.setLast(this.last);
		clone.setExtend(extend);
		return clone;
	}
	
	
	/**
	 * count the number of values different from NONE 
	 * @return the number of "significant" values
	 */
	public final synchronized int count() {
		return count;
	}
	
	
	
	/**
	 * Ensures that the array can be used with the given.
	 * If the capacity is inferior to the desired index, 
	 * the size of the array is dynamically extended. 
	 * @param index: the index for the new capacity
	 */
	protected synchronized void ensureCapacity(int index) {
		int current = ((values==null || values.length==0) ? 0 : values.length );
		if (index >= current) {		
			int ext=(this.extend >0 ? this.extend : DEFAULT_EXTEND);
			int capacity = ((index - current / ext ) + 1)
					* ext;
			int[] newvalues = new int[current + ext];
			clear(newvalues);
			if (current > 0) {
				System.arraycopy(values, 0, newvalues, 0,current);
			}
			values = newvalues;
		}
	}
	
	/**
	 * query the array to get the value stored at index
	 * @param index : the index of the value
	 * @return : the value of the array at the index
	 */
	public final synchronized int get(int index)  {
		if (index > last)
			throw new ArrayIndexOutOfBoundsException(index);
		return values[index];
	}
	
	
	/**
	 * Query the array to get it current block size for extension
	 * @return : the size of the extension
	 */
	public final synchronized int getExtend() {
		return extend;
	}
	/**
	 * gives index of a significant value
	 * @return Returns the last .
	 */
	public synchronized int last() {
		return last;
	}
	
	/**
	 * remove all occurence of the given element from the array
	 * @param element : the value to remove from the array
	 */
	public synchronized void remove(int element)  {
		if (values != null && values.length > 0) {
			for (int i = 0; i< last ;i++ ) 
				if (values[i]==element) {
					values[i]= NONE;
					count--;
				}
			while(values[last]==NONE && last >0) last--; 
		}
	}
	
	/**
	 * Remove the element stored between index from and index to
	 * @param indexFrom : the first index of element to be removed
	 * @param indexTo: the last index of element to be removed
	 * NOTE : The array values are shifted
	 */
	public synchronized void remove(int indexFrom, int indexTo )  {
		if (indexFrom > indexTo) {
			remove(indexTo, indexFrom);
		} else {
			if (values != null && values.length > 0) {
				int size = indexTo - indexFrom;
				if (indexFrom >= values.length - 1) {
					values[indexFrom]=NONE;
					size =1;
				} else {
					System.arraycopy(values,indexTo+1,values,indexFrom,values.length-1-indexTo);
					for(int i=values.length-1-size; i<values.length; i++)
						values[i]=NONE;
				}
				count-=size;
				last-=size;
			}
		}
	}
	
	
	/**
	 * Remove the element stored at index in the array
	 * @param index : the index of the removed 
	 * @see remove(int from , int to);
	 */
	public synchronized void removeAt(int index)  {
		remove(index, index);
	}
	
//	public synchronized void insert
	
	
	/**
	 * Store the given element at the given index
	 * @param index : the storage index
	 * @param element : the element to store
	 * @return : the previous value stored int the array (eventually NONE)
	 */
	public synchronized int set(int index, int element)  {
		ensureCapacity(index);
		int result = values[index];
		values[index] = element;
		if (element != NONE && result == NONE) count++;
		last = (index < last ? last : index);
		return result;
	}
	/**
	 * Set the number of "significant" values
	 * @param count The count to set.
	 */
	protected synchronized void setCount(int count) {
		this.count = (count <= 0 ? 0: count);
	}
	
	/**
	 * Set the block size for extension;
	 * @param extend : the new size for extension
	 */
	public final synchronized void setExtend(int extend) {
		this.extend = (extend <= 0 ? DEFAULT_EXTEND : extend);
	}
	/**
	 * Set the last index o significant values
	 * @param last The last to set.
	 */
	protected synchronized void setLast(int last) {
		this.last = last;
	}
	
	
	
	
	/**
	 * Set the content of the array and return the eventual previous content
	 * @param values : the new values
	 * @return the previous content (eventually null);
	 * NOTE : the content is entirely replaced
	 */
	protected synchronized int[] setValues(int[] values) {
		int [] result = this.values;
		this.values = values;
		update();
		return result;
	}
	
	
	/**
	 * Build a copy of the array limited to 
	 * the last element 
	 * @return the copied array
	 */
	public final synchronized int[] toArray() {
		if (values == null) return null;
		int[] result = new int[last+1];
		clear(result);
		System.arraycopy(values, 0, result, 0, last);
		return result;
	}
	
	
	/** 
	 * Convert the dynamic array into a string
	 * @see java.lang.Object#toString()
	 */
	public synchronized String toString() {
		StringBuffer buffer = new StringBuffer("DynamicIntArray@");
		buffer.append(Integer.toHexString(hashCode()));
		buffer.append("(");
		buffer.append("count=");
		buffer.append(count);
		buffer.append(",");
		buffer.append("last=");
		buffer.append(last);
		buffer.append(",");
		buffer.append("length=");
		buffer.append(values.length);
		buffer.append("):");
		buffer.append(toStringContent());
		return buffer.toString();
	}
	
	/**
	 * Convert the content of the array into a string
	 * @return the string corresponding to the content
	 */
	public synchronized String toStringContent() {
		StringBuffer buffer = new StringBuffer("[");
		boolean separator=false; 
		if (values != null && values.length > 0) { 
			for(int i=0; i<values.length;i++) {
				buffer.append((values[i] == NONE)? "X": ""+values[i]);
				if (i==last && !separator) {
					buffer.append("|");
					separator=true;
				}else{
					buffer.append(" ");
				}
			}
		}
		buffer.append("]");
		return buffer.toString();
	}
	
	
	/**
	 * re-evaluate count and last by walking through the values.
	 */
	protected final synchronized void update() {
		if (this.values == null || this.values.length == 0)
			setCount(0);
		else {
			int l = -1;
			int c=0;
			for (int i = this.values.length - 1; i >= 0; i--) {
				if (this.values[i] != NONE) {
					c++;
					l = (l ==-1 ? i : l);
				}
			}
			setLast(l);
			setCount(c);
			}
	}
	
	/**
	 * collect the NONE values
	 * NOTE : calling this method 
	 */
	public void shrink() {
		int start = -1;
		int size = -1;
		int last_save= last;
		for(int i= 0; i < last; i++) {
			if (values[i] == NONE) {
				if (start == -1) {
					start = i;
				}
			} else if (values[i] != NONE && start != -1 ) {
				System.arraycopy(values, i, values, start, last - i );
				clear(last - (i-start), last);
				last -= i-start;
				i--;
				start = -1;
			}
		}
	}
	/**
	 * @return Returns the values.
	 */
	public synchronized int[] getValues() {
		return values;
	}
}